from unittest.mock import patch

from django.test import SimpleTestCase

from manage_django_project.management.commands import pip_audit, shell
from manage_django_project.tests.cmd2_test_utils import BaseShellTestCase
from manage_django_project.tests.command_test_utils import call_command_capture_subprocess


class FakeNamedTemporaryFile:
    def __init__(self, prefix: str = 'prefix', suffix: str = 'suffix'):
        self.name = f'/tmp/{prefix}<rnd>{suffix}'

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass


class PipAuditShellTestCase(BaseShellTestCase):
    def test_pip_audit(self):
        with patch.object(shell, 'verbose_check_call') as call_mock:
            stdout, stderr = self.execute(command='pip_audit')
        self.assertEqual(stderr, '')
        self.assertIn('call command manage_django_project.pip_audit', stdout)
        call_mock.assert_called_once()


class PipAuditTestCase(SimpleTestCase):
    maxDiff = None

    def test_pip_audit(self):
        with patch('tempfile.NamedTemporaryFile', FakeNamedTemporaryFile):
            popenargs = call_command_capture_subprocess(cmd_module=pip_audit)
        self.assertEqual(
            popenargs,
            [
                [
                    '.../bin/uv',
                    'export',
                    '--no-header',
                    '--frozen',
                    '--no-editable',
                    '--no-emit-project',
                    '-o',
                    '/tmp/requirements<rnd>.txt',
                ],
                [
                    '.../bin/pip-audit',
                    '--strict',
                    '--require-hashes',
                    '--disable-pip',
                    '-v',
                    '-r',
                    '/tmp/requirements<rnd>.txt',
                ],
            ],
        )



import inspect
import sys
import unittest
from pathlib import Path

from django.core.management import call_command
from manageprojects.test_utils.subprocess import SubprocessCallMock

from manage_django_project.management.commands.coverage import erase_coverage_data
from manage_django_project.tests import PROJECT_ROOT


def get_rstrip_paths():
    return (
        Path(sys.executable).parent.parent,
        PROJECT_ROOT.parent,
    )


def call_command_capture_subprocess(cmd_module) -> list:
    assert inspect.ismodule(cmd_module)

    with SubprocessCallMock() as call_mock:
        call_command(cmd_module.Command())

    popenargs = call_mock.get_popenargs(rstrip_paths=get_rstrip_paths())
    return popenargs


class EraseCoverageDataMixin(unittest.TestCase):
    def setUp(self) -> None:
        super().setUp()
        erase_coverage_data.erased = False

    def tearDown(self) -> None:
        super().tearDown()
        erase_coverage_data.erased = False



import sys

from manage_django_project.management.base import BasePassManageCommand, ManageDjangoToolsExecutor
from manage_django_project.management.commands.coverage import coverage_combine_report, erase_coverage_data


class Command(BasePassManageCommand):
    help = 'Run tests via nox'

    def run_from_argv(self, argv):
        super().run_from_argv(argv)

        tools_executor = ManageDjangoToolsExecutor()

        try:
            # Just pass every argument to the origin nox CLI:
            tools_executor.verbose_check_call('nox', *argv[2:], exit_on_error=True)
            coverage_combine_report(verbose=True)
        finally:
            # Always remove coverage data files, after test run:
            erase_coverage_data(verbose=False)

        sys.exit(0)



from cli_base.cli_tools.subprocess_utils import ToolsExecutor
from django_rich.management import RichCommand
from rich.panel import Panel

from manage_django_project.config import project_info


class PrintHelpMixin:
    help = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        assert self.help is not None, 'Help must be set in sub class!'
        self._help_printed = False

    def print_help_once(self):
        if not self._help_printed:
            self.console.print(Panel(f'[bold]{self.help}'))

        self._help_printed = True


class BaseManageCommand(PrintHelpMixin, RichCommand):

    def execute(self, *args, **options):
        if options['verbosity']:
            self.print_help_once()

        project_info.assert_initialized()

        super().execute(*args, **options)


class BasePassManageCommand(PrintHelpMixin, RichCommand):
    help = ''

    def run_from_argv(self, argv):
        self.print_help_once()


class ManageDjangoToolsExecutor(ToolsExecutor):
    """
    Call tools from current Python /.venv/bin/ path.
    """

    def __init__(self):
        project_info.assert_initialized()
        project_root_path = project_info.config.project_root_path
        super().__init__(cwd=project_root_path)



from unittest.mock import patch

from bx_py_utils.environ import OverrideEnviron
from django.test import SimpleTestCase
from manageprojects.test_utils.subprocess import SubprocessCallMock

from manage_django_project.management.commands import nox, shell
from manage_django_project.tests.cmd2_test_utils import BaseShellTestCase
from manage_django_project.tests.command_test_utils import EraseCoverageDataMixin, get_rstrip_paths


class NoxShellTestCase(BaseShellTestCase):
    def test_basic_nox(self):
        with patch.object(shell, 'verbose_check_call') as call_mock:
            stdout, stderr = self.execute(command='nox')
        self.assertEqual(stderr, '')
        self.assertIn('call command manage_django_project.nox', stdout)
        call_mock.assert_called_once()


class NoxTestCase(EraseCoverageDataMixin, SimpleTestCase):
    maxDiff = None

    def test_basic_nox(self):
        command = nox.Command()
        with (
            SubprocessCallMock() as call_mock,
            OverrideEnviron(
                nox_ENV_NAME='foobar',  # env variable used to "detect" nox run
            ),
        ):
            try:
                command.run_from_argv(argv=['--foo', '--bar'])
            except SystemExit as err:
                self.assertEqual(err.code, 0)

        popenargs = call_mock.get_popenargs(rstrip_paths=get_rstrip_paths())
        self.assertEqual(
            popenargs,
            [
                ['.../bin/nox'],
                ['.../bin/coverage', 'combine', '--append'],
                ['.../bin/coverage', 'report'],
                ['.../bin/coverage', 'xml'],
                ['.../bin/coverage', 'json'],
                ['.../bin/coverage', 'erase'],
            ],
        )




from manage_django_project.management.base import BaseManageCommand, ManageDjangoToolsExecutor


class Command(BaseManageCommand):
    help = 'Just install the project as editable via pip (Useful if version has been changed)'

    def handle(self, *args, **options):
        tools_executor = ManageDjangoToolsExecutor()
        tools_executor.verbose_check_call('pip', 'install', '--no-deps', '-e', '.')



from bx_py_utils.test_utils.unittest_utils import BaseDocTests

import manage_django_project
import manage_django_project_example


class DocTests(BaseDocTests):
    def test_doctests(self):
        self.run_doctests(
            modules=(manage_django_project, manage_django_project_example),
        )



import django
from bx_py_utils.test_utils.snapshot import assert_text_snapshot

from manage_django_project.tests.cmd2_test_utils import BaseShellTestCase


class ManageDjangoShellTestCase(BaseShellTestCase):
    def test_help(self):
        stdout, stderr = self.execute(command='help')
        self.assertEqual(stderr, '')
        self.assertIn('Documented commands', stdout)
        self.assertIn('django.core', stdout)
        self.assertIn('makemessages', stdout)
        self.assertIn('makemigrations', stdout)
        self.assertIn('manage_django_project', stdout)
        self.assertIn('run_dev_server', stdout)

        assert_text_snapshot(
            got=stdout,
            snapshot_name=f'test_command_shell_help_django{".".join(str(x) for x in django.VERSION[:2])}',
        )



from cli_base.cli_tools.test_utils.rich_test_utils import NoColorEnvRichClick
from django.test import SimpleTestCase

from manage_django_project.tests.test_manage import call_manage_py


class TestCase(SimpleTestCase):
    maxDiff = None

    def test_happy_path(self):
        with NoColorEnvRichClick(width=80):
            output = call_manage_py('playwright', '--help')

        self.assertIn('manage_django_project_example playwright --help\n', output)
        self.assertIn('Usage: playwright [options] [command]\n', output)
        self.assertIn('Call playwright CLI', output)
        self.assertIn('install [options] [browser...]', output)



from django.test import SimpleTestCase

from manage_django_project.management.commands import coverage
from manage_django_project.tests.command_test_utils import EraseCoverageDataMixin, call_command_capture_subprocess


class CoverageTestCase(EraseCoverageDataMixin, SimpleTestCase):
    maxDiff = None

    def test_happy_path(self):
        popenargs = call_command_capture_subprocess(cmd_module=coverage)
        self.assertEqual(
            popenargs,
            [
                ['.../bin/coverage', 'run'],
                ['.../bin/coverage', 'combine', '--append'],
                ['.../bin/coverage', 'report'],
                ['.../bin/coverage', 'xml'],
                ['.../bin/coverage', 'json'],
                ['.../bin/coverage', 'erase'],
            ],
        )



from __future__ import annotations

import dataclasses
import inspect
import io

from cmd2 import CommandResult
from cmd2.ansi import strip_style
from cmd2_ext_test import ExternalTestMixin
from django.core.management import BaseCommand
from django.test import SimpleTestCase
from django_tools.unittest_utils.stdout_redirect import DenyStdWrite

from manage_django_project.management.commands.shell import ManageDjangoProjectApp


class ManageDjangoTestApp(ExternalTestMixin, ManageDjangoProjectApp):
    pass


class BaseShellTestCase(SimpleTestCase):
    """
    Base class for cmd2 app test cases
    """

    maxDiff = 2000

    def setUp(self):
        super().setUp()
        self.app = ManageDjangoTestApp()
        self.app.fixture_setup()

    def tearDown(self):
        super().tearDown()
        self.app.fixture_teardown()

    def execute(self, command, remove_colors=True, rstrip=True):
        out = self.app.app_cmd(command)

        assert isinstance(out, CommandResult)
        if out.stdout is None:
            stdout = ''
        else:
            stdout = str(out.stdout)
            if remove_colors:
                stdout = strip_style(stdout)
            if rstrip:
                stdout = '\n'.join(line.rstrip() for line in stdout.splitlines())

        if out.stderr is None:
            stderr = ''
        else:
            stderr = str(out.stderr)
            if remove_colors:
                stderr = strip_style(stderr)

        return stdout, stderr


class Buffer(io.StringIO):
    def __repr__(self):
        return '<captured_call_command StringIO buffer>'


@dataclasses.dataclass
class Result:
    exit_code: int | None
    stdout: str
    stderr: str


def run_command_from_argv(command, argv: list) -> Result:
    """
    Call django manage command via `run_from_argv()` and return stdout + stderr
    """
    try:
        assert inspect.ismodule(command)
        CommandClass = command.Command
        assert issubclass(CommandClass, BaseCommand)
    except Exception as err:
        raise AssertionError(f'{command!r} is no Django Management command: {err}')

    command_name = command.__name__
    command_name = command_name.rsplit('.', 1)[-1]

    capture_stdout = Buffer()
    capture_stderr = Buffer()

    command_instance = CommandClass(
        stdout=capture_stdout,
        stderr=capture_stderr,
        no_color=True,
        force_color=False,
    )

    with DenyStdWrite(name=command_name):
        try:
            command_instance.run_from_argv(argv)
        except SystemExit as err:
            exit_code = err.code
        else:
            exit_code = None

    return Result(
        exit_code=exit_code,
        stdout=capture_stdout.getvalue(),
        stderr=capture_stderr.getvalue(),
    )



import sys

from manage_django_project.management.base import BasePassManageCommand


class Command(BasePassManageCommand):
    help = 'Call playwright CLI'

    def run_from_argv(self, argv):
        super().run_from_argv(argv)

        try:
            from playwright.__main__ import main
        except ImportError as err:
            self.stderr.write(f'No playwright installed? (Origin error: {err})')
        else:
            sys.argv = sys.argv[1:]
            main()



import os
import unittest.util
from pathlib import Path

import django
from bx_py_utils.test_utils.deny_requests import deny_any_real_request
from rich import print
from typeguard import install_import_hook

import manage_django_project
from manage_django_project.config import project_info


# Check type annotations via typeguard in all tests:
install_import_hook(packages=('manage_django_project', 'manage_django_project_example'))


PROJECT_ROOT = Path(manage_django_project.__file__).parent.parent


def pre_configure_tests() -> None:
    print(f'Configure unittests via "load_tests Protocol" from {Path(__file__).relative_to(Path.cwd())}')

    # Hacky way to display more "assert"-Context in failing tests:
    _MIN_MAX_DIFF = unittest.util._MAX_LENGTH - unittest.util._MIN_DIFF_LEN
    unittest.util._MAX_LENGTH = int(os.environ.get('UNITTEST_MAX_LENGTH', 300))
    unittest.util._MIN_DIFF_LEN = unittest.util._MAX_LENGTH - _MIN_MAX_DIFF

    # Deny any request via docket/urllib3 because tests they should mock all requests:
    deny_any_real_request()


def init_django4unittests() -> None:
    """
    Make it possible to run tests via:
      .venv/bin/python -m unittest
    by setup Django with test settings
    """
    project_info.initialize()

    DJANGO_SETTINGS_MODULE: str = project_info.config.test_settings
    print(f'Force {DJANGO_SETTINGS_MODULE=}')
    os.environ['DJANGO_SETTINGS_MODULE'] = DJANGO_SETTINGS_MODULE

    django.setup()


def load_tests(loader, tests, pattern):
    """
    Use unittest "load_tests Protocol" as a hook to setup test environment before running tests.
    https://docs.python.org/3/library/unittest.html#load-tests-protocol
    """
    pre_configure_tests()
    init_django4unittests()
    return loader.discover(start_dir=Path(__file__).parent, pattern=pattern)



from bx_py_utils.environ import OverrideEnviron
from cli_base.cli_tools.dev_tools import is_verbose
from cli_base.cli_tools.test_utils.snapshot import UpdateTestSnapshotFiles

from manage_django_project.config import project_info
from manage_django_project.management.commands.nox import Command as BaseNoxCommand


class Command(BaseNoxCommand):
    help = 'Update all snapshot files (by remove and recreate all snapshot files)'

    def run_from_argv(self, argv):
        self.print_help_once()

        root_path = project_info.config.project_root_path
        verbose = is_verbose(argv=argv)

        with UpdateTestSnapshotFiles(root_path=root_path, verbose=verbose), OverrideEnviron(RAISE_SNAPSHOT_ERRORS='0'):
            # Run "nox" to recreate all snapshot files:
            super().run_from_argv(argv)



from cli_base.run_pip_audit import run_pip_audit
from django_rich.management import RichCommand

from manage_django_project.config import project_info


class Command(RichCommand):
    help = 'Run `pip-audit` with configuration from `pyproject.toml`'

    def handle(self, *args, **options):
        self.console.print(f'\n[bold]{self.help}')

        project_root_path = project_info.config.project_root_path

        run_pip_audit(verbosity=options['verbosity'], base_path=project_root_path)



from unittest.mock import patch

from bx_py_utils.test_utils.redirect import RedirectOut
from django.test import SimpleTestCase

from manage_django_project.management.commands import shell, update_req
from manage_django_project.tests.cmd2_test_utils import BaseShellTestCase
from manage_django_project.tests.command_test_utils import call_command_capture_subprocess


class UpdateReqShellTestCase(BaseShellTestCase):
    def test_basic_update_req(self):
        with patch.object(shell, 'verbose_check_call') as call_mock:
            stdout, stderr = self.execute(command='update_req')
        self.assertEqual(stderr, '')
        self.assertIn('call command manage_django_project.update_req', stdout)
        call_mock.assert_called_once()


class UpdateReqTestCase(SimpleTestCase):
    maxDiff = None

    def test_basic_update_req(self):
        with RedirectOut() as buffer:
            popenargs = call_command_capture_subprocess(cmd_module=update_req)

        self.assertEqual(buffer.stderr, '')
        self.assertIn('Update project requirements via uv', buffer.stdout)

        # Check samples:
        self.assertEqual(popenargs[0], ['.../bin/pip', 'install', '-U', 'pip'])
        self.assertEqual(popenargs[-1], ['.../bin/pre-commit', 'autoupdate'])



import shutil
from unittest import TestCase

from bx_py_utils.path import assert_is_file
from cli_base.cli_tools.subprocess_utils import verbose_check_output

from manage_django_project import __version__
from manage_django_project.tests import PROJECT_ROOT


def call_manage_py(*args) -> str:
    assert_is_file(PROJECT_ROOT / 'manage.py')

    system_python_bin = shutil.which('python3')
    assert 'python' in system_python_bin, f'{system_python_bin=}'

    output = verbose_check_output(system_python_bin, 'manage.py', *args, verbose=False, cwd=PROJECT_ROOT)
    return output


class ManageTestCase(TestCase):
    def test_manage_py_call(self):
        output = call_manage_py('--version')
        self.assertIn(__version__, output)  # Check Django version

        output = call_manage_py('--help')
        self.assertIn('.venv/bin/manage_django_project_example --help', output)
        self.assertIn('manage_django_project_example help <subcommand>', output)
        self.assertIn('Available subcommands:', output)
        self.assertIn('[django]', output)
        self.assertIn('[manage_django_project]', output)
        self.assertIn('project_info', output)



import inspect
import logging
import os
import tempfile
from pathlib import Path

from bx_py_utils.path import assert_is_file
from django.test import SimpleTestCase

import manage_django_project_example
from manage_django_project.config import (
    ManageConfig,
    NoPyprojectTomlFound,
    detect_pyproject_toml,
    get_config,
    project_info,
    read_pyproject_toml,
)
from manage_django_project.exceptions import ConfigKeyError, ModuleNotFound, SettingsNotFound


logger = logging.getLogger(__name__)


class TemporaryWorkDirectory:
    def __init__(self, prefix=None):
        self.prefix = prefix
        self.old_cwd = Path.cwd()

    def __enter__(self) -> Path:
        temp_name = tempfile.mkdtemp(prefix=self.prefix)
        self.temp_path = Path(temp_name)
        os.chdir(self.temp_path)
        return self.temp_path

    def __exit__(self, exc_type, exc_val, exc_tb):
        os.chdir(self.old_cwd)


class ConfigTestCase(SimpleTestCase):
    def test_basic(self):
        project_info.assert_initialized()

        self.assertEqual(project_info.distribution_name, 'manage_django_project')
        self.assertIsInstance(project_info.config, ManageConfig)
        self.assertEqual(project_info.config.module, manage_django_project_example)

    def test_detect_pyproject_toml(self):
        pyproject_toml_path = detect_pyproject_toml()
        assert_is_file(pyproject_toml_path)
        self.assertEqual(pyproject_toml_path.name, 'pyproject.toml')

        with TemporaryWorkDirectory(prefix='test_detect_pyproject_toml') as temp_path:
            with self.assertRaisesMessage(NoPyprojectTomlFound, f'No "pyproject.toml" found from cwd {temp_path}'):
                detect_pyproject_toml()

    def test_read_pyproject_toml(self):
        pyproject_toml = read_pyproject_toml(path=project_info.pyproject_toml_path)
        self.assertIsInstance(pyproject_toml, dict)
        self.assertEqual(pyproject_toml['project']['name'], 'manage_django_project')

    def test_manage_config(self):
        with self.assertRaisesMessage(
            SettingsNotFound,
            'Settings "wrong_local_settings" can not import: Not found!'
            ' (Hint: Check "local_settings" in your pyproject.toml !)',
        ):
            ManageConfig(
                module=project_info.config.module,
                project_root_path=project_info.config.project_root_path,
                local_settings='wrong_local_settings',
                test_settings=project_info.config.test_settings,
            )
        with self.assertRaisesMessage(
            SettingsNotFound,
            'No module named \'notexisting\' (Hint: Check "test_settings" in your pyproject.toml !)',
        ):
            ManageConfig(
                module=project_info.config.module,
                project_root_path=project_info.config.project_root_path,
                local_settings=project_info.config.local_settings,
                test_settings='notexisting.foo.bar',
            )

    def test_get_config(self):
        config = get_config()
        self.assertIsInstance(config, ManageConfig)
        self.assertEqual(config, project_info.config)

        with TemporaryWorkDirectory(prefix='test_get_config') as temp_path:
            test_toml = temp_path / 'pyproject.toml'
            test_toml.touch()

            with self.assertRaisesMessage(AssertionError, f'No data from: {test_toml}'):
                get_config()

            test_toml.write_text(
                inspect.cleandoc(
                    '''
                    [foo]
                    bar=1
                    '''
                )
            )

            with self.assertRaisesMessage(ConfigKeyError, "No 'manage_django_project' "):
                get_config()

            test_toml.write_text(
                inspect.cleandoc(
                    '''
                    [manage_django_project]
                    bar=1
                    '''
                )
            )
            with self.assertRaisesMessage(ConfigKeyError, "No 'module_name' "):
                get_config()

            test_toml.write_text(
                inspect.cleandoc(
                    '''
                    [manage_django_project]
                    module_name="foobar"
                    local_settings="foobar"
                    test_settings="foobar"
                    '''
                )
            )
            with self.assertRaisesMessage(ModuleNotFound, "No module named 'foobar' (Hint: Check module_name"):
                get_config()



from unittest.mock import patch

from django.test import SimpleTestCase
from manageprojects.test_utils.subprocess import SubprocessCallMock

from manage_django_project.management.commands import shell, update_test_snapshot_files
from manage_django_project.tests.cmd2_test_utils import BaseShellTestCase
from manage_django_project.tests.command_test_utils import EraseCoverageDataMixin, get_rstrip_paths


class ShellTestCase(BaseShellTestCase):
    def test_update_test_snapshot_files(self):
        with patch.object(shell, 'verbose_check_call') as call_mock:
            stdout, stderr = self.execute(command='update_test_snapshot_files')
        self.assertEqual(stderr, '')
        self.assertIn('call command manage_django_project.update_test_snapshot_files', stdout)
        call_mock.assert_called_once()


class CallTestCase(EraseCoverageDataMixin, SimpleTestCase):
    maxDiff = None

    def test_update_test_snapshot_files(self):
        command = update_test_snapshot_files.Command()

        class ProjectInfoMock:
            def __init__(self):
                self.config = self
                self.project_root_path = self

                self.rglob_calls = []
                self.unlink_calls = 0

            def rglob(self, pattern):
                self.rglob_calls.append(pattern)
                return [self]

            def unlink(self):
                self.unlink_calls += 1

        project_info_mock = ProjectInfoMock()
        with SubprocessCallMock() as call_mock, patch(
            'manage_django_project.management.commands.update_test_snapshot_files.project_info', project_info_mock
        ):
            try:
                command.run_from_argv(argv=[])
            except SystemExit as err:
                self.assertEqual(err.code, 0)

        self.assertEqual(
            project_info_mock.rglob_calls,
            [
                '*.snapshot.*',  # Collect for deleting
                '*.snapshot.*',  # Collect for info print
            ],
        )
        self.assertEqual(project_info_mock.unlink_calls, 1)

        # Just normal nox calls:
        popenargs = call_mock.get_popenargs(rstrip_paths=get_rstrip_paths())
        self.assertEqual(
            popenargs,
            [
                ['.../bin/nox'],
                ['.../bin/coverage', 'combine', '--append'],
                ['.../bin/coverage', 'report'],
                ['.../bin/coverage', 'xml'],
                ['.../bin/coverage', 'json'],
                ['.../bin/coverage', 'erase'],
            ],
        )



from cli_base.cli_tools.dev_tools import coverage_combine_report, erase_coverage_data

from manage_django_project.management.base import BaseManageCommand, ManageDjangoToolsExecutor


class Command(BaseManageCommand):
    help = 'Run tests with coverage and report'

    def add_arguments(self, parser):
        parser.add_argument(
            "--context",
            help='The context label to record for this coverage run.',
        )

    def handle(self, *args, **options):
        verbose = options['verbosity'] > 0

        tools_executor = ManageDjangoToolsExecutor()

        old_coverage_data = tools_executor.cwd / '.coverage'
        old_coverage_data.unlink(missing_ok=True)

        args = ['coverage', 'run']
        if context := options['context']:
            args.append('--context')
            args.append(context)

        try:
            tools_executor.verbose_check_call(*args, verbose=verbose, exit_on_error=True)
            coverage_combine_report(verbose=True)
        finally:
            # Always remove coverage data files, after test run:
            erase_coverage_data(cwd=tools_executor.cwd, verbose=verbose)



from cli_base.run_pip_audit import run_pip_audit
from django_rich.management import RichCommand

from manage_django_project.config import project_info
from manage_django_project.management.base import ManageDjangoToolsExecutor


class Command(RichCommand):
    help = 'Update project requirements via uv'

    def handle(self, *args, **options):
        self.console.print(f'\n[bold]{self.help}')
        verbosity = options['verbosity']

        project_root_path = project_info.config.project_root_path

        tools_executor = ManageDjangoToolsExecutor()

        tools_executor.verbose_check_call('pip', 'install', '-U', 'pip')
        tools_executor.verbose_check_call('pip', 'install', '-U', 'uv')
        tools_executor.verbose_check_call('uv', 'lock', '--upgrade')

        run_pip_audit(base_path=project_root_path, verbosity=verbosity)

        # Install new dependencies in current .venv:
        tools_executor.verbose_check_call('uv', 'sync')

        if tools_executor.is_executable('pre-commit'):
            # Update git pre-commit hooks:
            tools_executor.verbose_check_call('pre-commit', 'autoupdate')



import dataclasses
import logging
import tomllib
from importlib import import_module
from importlib.util import find_spec
from pathlib import Path
from types import ModuleType

from bx_py_utils.path import assert_is_file
from packaging.version import Version

from manage_django_project.exceptions import ConfigKeyError, ModuleNotFound, SettingsNotFound


logger = logging.getLogger(__name__)


@dataclasses.dataclass
class ManageConfig:
    module: ModuleType
    project_root_path: Path

    # dotted name of Django settings for...
    local_settings: str  # ... run local Django dev. server
    test_settings: str  # ... run tests

    # All Django manage commands that should use the "test" settings, instead of "local" settings:
    test_settings_commands: tuple = ('test', 'coverage', 'nox', 'update_test_snapshot_files')

    def __post_init__(self):
        for attr_name in ('local_settings', 'test_settings'):
            settings_str = getattr(self, attr_name)
            try:
                spec = find_spec(settings_str)
                assert spec, f'Settings "{settings_str}" can not import: Not found!'
            except (ModuleNotFoundError, AssertionError) as err:
                raise SettingsNotFound(f'{err} (Hint: Check "{attr_name}" in your pyproject.toml !)')


def read_pyproject_toml(path: Path) -> dict:
    assert_is_file(path)
    pyproject_toml = tomllib.loads(path.read_text(encoding='UTF-8'))
    return pyproject_toml


class NoPyprojectTomlFound(FileNotFoundError):
    pass


def detect_pyproject_toml() -> Path:
    path = Path.cwd()
    for _ in range(100):
        candidate = path / 'pyproject.toml'
        if candidate.is_file():
            return candidate

        if len(path.parts) > 1:
            # Look in parent direcotry
            path = path.parent
        else:
            # we are in root directory -> don't look further
            break

    raise NoPyprojectTomlFound(f'No "pyproject.toml" found from cwd {Path.cwd()}')


def get_config() -> ManageConfig:
    """
    Init and return ManageConfig from settings in pyproject.toml
    """
    pyproject_toml_path = detect_pyproject_toml()
    pyproject_toml = read_pyproject_toml(path=pyproject_toml_path)
    assert pyproject_toml, f'No data from: {pyproject_toml_path}'

    try:
        toml_cfg = pyproject_toml['manage_django_project']
        module_name = toml_cfg['module_name']
        local_settings = toml_cfg['local_settings']
        test_settings = toml_cfg['test_settings']
    except KeyError as err:
        raise ConfigKeyError(f'No {err} in {pyproject_toml_path}')

    assert module_name, f'Config value "module_name" is empty in {pyproject_toml_path}'
    assert local_settings, f'Config value "local_settings" is empty in {pyproject_toml_path}'
    assert test_settings, f'Config value "test_settings" is empty in {pyproject_toml_path}'

    try:
        module = import_module(module_name)
    except Exception as err:
        raise ModuleNotFound(f'{err} (Hint: Check {module_name=} in {pyproject_toml_path}')

    config = ManageConfig(
        module=module,
        project_root_path=pyproject_toml_path.parent,
        local_settings=local_settings,
        test_settings=test_settings,
    )
    return config


@dataclasses.dataclass
class ProjectInfo:
    initialized: bool = False

    config: ManageConfig = None
    pyproject_toml_path: Path | None = None
    distribution_name: str | None = None
    module_version: Version | None = None

    def initialize(self):
        if self.initialized:
            logger.info('Already initialized')
            return

        self.config = get_config()

        self.module_version = Version(self.config.module.__version__)
        self.pyproject_toml_path = Path(self.config.project_root_path, 'pyproject.toml')

        pyproject_toml = self.get_pyproject_toml()
        self.distribution_name = pyproject_toml['project']['name']

        self.initialized = True

    def assert_initialized(self):
        assert self.initialized is True, f'Not initialized: {self}'

    def get_pyproject_toml(self) -> dict:
        return read_pyproject_toml(path=self.pyproject_toml_path)

    def get_settings_by_command(self, *, command_name) -> str:
        if command_name in self.config.test_settings_commands:
            return self.config.test_settings

        return self.config.local_settings

    def get_current_settings(self, argv) -> str | None:
        for arg in argv:
            if arg.startswith('--settings'):
                # e.g.: Start a manage command with --settings option -> don't force any settings
                return

        if len(argv) > 1:
            settings_name = self.get_settings_by_command(command_name=argv[1])
        else:
            settings_name = self.config.local_settings

        return settings_name


project_info = ProjectInfo()



"""
    manage_django_project
    Helper to develop Django projects.
"""

# See https://packaging.python.org/en/latest/specifications/version-specifiers/
__version__ = '0.9.2'
__author__ = 'Jens Diemer <git@jensdiemer.de>'
